function Impuls = impulres(A,B,P,Time,D,rhop,etap,x,Taylor,entr,alpha)

% this function computes impulse responses to 1% shock (i.e., .01)
% Input:
% A is the feedback matrix
% B is the feedforward matrix
% P is the matrix computed in solveb and describes the evolution of shocks
% T is the number of periods that are of interest 
% 
% Output:
% responses for each shock (i.e. [response to shock 1
%                                 response to shock 2
%                                 ....               
%                                 ...                ]
if entr ==0  %we have made a change of variable in makealphabeta in order to impose information restrictions. We have now to undue the change
    Q = eye(25);
    if size(Q,1)>25 | size(Q,1)<25
        error('the size of Q and A is different')
    end
    Q(18,18) = 1-alpha;
    Q(18,8) = 1;
    QI=inv(Q);  
end

Impuls=[];
zxz=size(P,1);
[n1,N]=size(D);

if zxz ~= 2*n1
    error('fatal (impulres) parameters of D and P don''t match properly')
end

for i=1:N
    eps0 = zeros(zxz,1);
    epsbar = zeros(N,1);
    
    if i == 16;%this is a permanent technology shock
        epsbar(i)=.0012;
    elseif i == 17;%this is a gamma shock
        epsbar(i)= -.01;
    elseif i == 19;%this is a monetary policy shock    
        if Taylor == 0
            epsbar(i)=.11;
            %epsbar(i)=((1-rhop)/(1+etap))/x;
            %this is a perturbation to xhat(t). if g(t)=M(t+1)/M(t), then ghat(t) is the percent change (in decimal terms)
        %in the monetary base. We have that ghat(t)=(x/(1+x))*xhat(t). In the handbook chapter it is reported that
        %the impact of a monetary policy shock on the base is 0.35 percent in annual terms. This means that 400*ghat(t)=.35. 
        %to figure out the required shock to xhat(t), solve xhat(t)=0.35*(1+x)/(400*x). with x=.0079, this corresponds to
        %xhat(t)=0.11. The impulse response graph program assumes that things have to be multiplied by 100 to convert 
        %into percent terms.
        end
        if Taylor == 1 | Taylor ==2 
            epsbar(i)=0.3;
        end 
    elseif i == 21%this is a sigma shock
        epsbar(i) = .1;
    elseif i ~= 16 & i ~= 17 & i ~= 19 & i ~= 21 & i~=14 & i~=8 & i ~= 12
        epsbar(i) = .01;
    elseif i == 14%this is a v shock
        epsbar(i) = .36;
    elseif i == 8%this is a theta shock
        epsbar(i) = 0.01; %-.2067;
    elseif i==12;%this is a zeta shock
        epsbar(i)=.06;
    end
    
    eps0=[D*epsbar; zeros(3*N,1)];
    j=(i-1)*3+1;
    
    shock(:,[1])=eps0;
    z1(:,[1])=B*eps0;
    
    if entr ==1 | entr==-1
        Resp=[z1(:,[1]); shock(j,[1])]; 
    elseif entr == 0
        Resp=[QI*z1(:,[1]); shock(j,[1])];     
    end
    
    for t=2:Time,
        shock(:,[t])=P*shock(:,[t-1]);
        z1(:,[t])=A*z1(:,[t-1])+B*shock(:,[t]);
        
        if entr == 1 |entr==-1
            Respp=[z1(:,[t]);shock(j,[t])];
        elseif entr ==0
            Respp=[QI*z1(:,[t]);shock(j,[t])];            
        end
        
        Resp=[Resp,Respp]; 
    end
    Impuls=[Impuls;Resp];
end
   
    