function [ll]=mykalmandif(data,T,Z,R,Qt,H,xini,Pini,Pinf);

%Diffuse Kalman filter. 
%Based on S.J.Koopman and J.Durbin "Filtering and Smoothing of State Vector for Diffuse State-Space Models", in Journal of
%Time Series Analysis, Vol. 24(1), pp. 85-98
%Written by Gianni Lombardo (ECB), February 2005

bigt=size(data,2);

% LL=zeros(size(T,1),size(T,1),bigt);
% VV=LL;
% NN=LL;
Zt=Z';
Tt=T';

n=size(data,1);
Q=R*Qt*R';

stv=xini;
P=Pini;
% use the following if want to use conditional variance
%the starting point and the number of iterations might be crucial for the location of 
% the likelihood since more persistent processes will be more far off their
% ergodic variance than non-persistent processes
% if the process is non-stationary that use ``priors''. Hammilton and
% Harvey suggest using kappa*I where kappa should be large to reflect a
% diffuse prior. Yet non-stable processes might not satisfy some conditions
% (see Harvey ch.4). For example if the system is stable than the initial
% condition should not matter anymore.
%dd=[];

% initial condition (last equation page 378 of Hamilton)
%P=inv(eye(size(T,1)^2)-kron(T,T))*Q(:);
%P=vec_(P,size(T,1));%,size(T,1)
Finf=Z*Pinf*Zt; 

if max(max(abs(Pinf)))==0 | min(min(Finf))<=1e-10
    P=0;
    for j=1:100
        P=T*P*Tt+Q;
    end
end


ll =(n*bigt/2)*log(2*pi);% negative of costant part of likelihood times #series*lenght
scfc=1;

% diffusion

gg=1;

dtinf=(det(Finf));
critval=1;
while critval>=1e-6 & dtinf>1e-15 & gg<bigt
    Pold=Pinf;
    
    if min(min(Finf))>1e-10 
        if rcond(Finf)>1e-15;
            invFinf=inv(Finf);
        else
            invFinf=pinv(Finf);
        end
        Kinf=T*Pinf*Zt*invFinf;
        LLinf=T-Kinf*Z;
        Ft=Z*P*Zt+H;
        
        % start the filter
        Kt=(T*P*Zt-Kinf*Ft)*invFinf; %Kalman gain (see Hamilton p. 381)
        ut=data(:,gg)-(Z*stv);% forecast error
        
        ll=ll+0.5*log(dtinf);  
        %filter updating     
        stv=T*stv+Kinf*ut; 
        %LL=T-Kt*Z;
        LLinft=LLinf';
        Pinf=T*Pinf*LLinft;
        P=T*P*LLinft-Kinf*Finf*Kt'+Q; %update of MSE of projection of state variables 
        gg=gg+1;
        
    else %if Finf=0
        % observables' forecast error
        Ft=Z*P*Zt+H;
        %if abs(imag(Ft))>1e-15
        %   warnign('complex covariance')
        %end
        
        % if rcond(Ft)<1e-15
        %warning('==========>>>>>>>>>>badly conditioned FF matrix')
        %ll=1e12
        if rcond(Ft)>1e-15;
            invFt=inv(Ft);
        else
            invFt=pinv(Ft);
        end
        %On pinv see Harvey's Forecasting ...p. 106 footnote where pinv is also suggested  
        % see also Kohn and Ansley (Biometrica 1983 v.70
        % p.683-688). 
        %return
        %end
        
        ut=data(:,gg)-(Z*stv);% forecast error         
        ll = ll + (0.5)*(log((det(Ft)))+ut'*(invFt)*ut);%likelihood function          
        %%%
        % start the filter
        Kt=T*P*Z'*invFt; %Kalman gain (see Hamilton p. 381)
        stv=T*stv+Kt*ut; 
        LL=T-Kt*Z;
        Pinf=T*Pinf*Tt;
        P=T*P*LL'+Q; %update of MSE of projection of state variables
        gg=gg+1;
    end %if F=0;
    critval=max(max(abs(Pinf)-abs(Pold)));
    Finf=Z*Pinf*Zt;
    dtinf=det(Finf);
end %while
for ii=gg:bigt
    % observables' forecast error
    Ft=Z*P*Zt+H;
    %if abs(imag(Ft))>1e-15
    %   warnign('complex covariance')
    %end
    
    
    % if rcond(Ft)<1e-15
    %warning('==========>>>>>>>>>>badly conditioned FF matrix')
    %ll=1e12;
    invFt=pinv(Ft);% see Harvey's Forecasting ...p. 106 footnote where pinv is also suggested  
    % see also Kohn and Ansley (Biometrica 1983 v.70
    % p.683-688). 
    %return
    %end
    
    ut=data(:,ii)-(Z*stv);% forecast error
    ll = ll + (0.5)*(log((det(Ft)))+ut'*(invFt)*ut);%likelihood function
    
    %%%
    % start the filter
    Kt=T*P*Z'*invFt; %Kalman gain (see Hamilton p. 381)
    stv=T*stv+Kt*ut; 
    LL=T-Kt*Z;
    P=T*P*LL'+Q; %update of MSE of projection of state variables
end

if abs(imag(ll)) > 0
    ll = 1e12;
end 

if isnan(ll)
    %disp('NAN loglik')
    ll=1e12;
end