function [edf] = financial(sigma_SS,omegabar,Rk,Re,k,n,Pe_SS,mu)

syms sig Zp1 Rkp1 Rep1 lev Petil omega sp1 ZminusR

%note: normcdf(x) = 0.5*erfc(-x/sqrt(2)), erfc = 1-erf

%opt controls which variables are used to expand the financial equations.
%opt = 1: sig omega Rkp1 Rep1 lev Petil (Petil is the 'average' measure of the premium)
%opt = 0: sig Zp1 Rkp1 Rep1 lev Petil
%opt = 2: sig sp1 lev ZminusR (ZminusR is the 'correct' measure of the premium)

%1+ZminusR is the ratio of the gross interest rate paid by entrepreneurs to the gross interest rate paid to households 

opt = 2;

if opt == 0
    omega   =   (Zp1/(1+Rkp1))*(lev-1)/lev;
elseif opt == 2
    omega   =   ((ZminusR+1)/sp1)*(lev-1)/lev;
end

z       =   (log(omega)+sig^2/2)/sig;
G       =   .5*(1-erf(-(z-sig)/sqrt(2)));
%F       =   normcdf(z);
F       =   .5*(1-erf(-z/sqrt(2)));
%Gamma   =   normcdf(z-sig)+omega*(1-normcdf(z));
Gamma   =   G+omega*(1-F);
Gam_muG =   (1-mu)*G+omega*(1-F);
Fprime  =   (1/sqrt(2*3.14159265358979))*exp(-z^2/2)/(omega*sig);
if opt ~= 2;sp1=(1+Rkp1)/(1+Rep1);end
mutimesG=   Petil/((1+Rkp1)*lev/(lev-1)); 

ff(1,1) =   (1-Gamma) * sp1 + ( sp1*Gam_muG-1 )/( 1 - mu*omega*Fprime/(1-F) );

ff(2,1) =   Gam_muG*lev*sp1-lev+1;

ff(3,1) =   mu*G*sp1*((1+Rep1)*(lev/(lev-1))) - Petil;

if opt == 1
    omega=omegabar;
elseif opt == 2
    sp1_SS=(1+Rk)/(1+Re);
    sp1=sp1_SS;
end

Rkp1=Rk;
Rep1=Re;
lev_SS=k/n;
lev=lev_SS;
Petil=Pe_SS;
Z_SS=(1+Rk)*omegabar*lev_SS/(lev_SS-1);
ZminusR_SS=Z_SS/(1+Re)-1;
ZminusR=ZminusR_SS;
Zp1=Z_SS;
sig=sigma_SS;
gg=eval(ff);

if abs(gg) > .1e-10 
    error('fatal (financial) not a steady state')
end

%
%sig Zp1 Rkp1 Rep1 lev Petil
%sig sp1 lev ZminusR

df(1,1)=diff(ff(1),'sig')*sigma_SS; df(2,1)=diff(ff(2),'sig')*sigma_SS;  df(3,1)=diff(ff(3),'sig')*sigma_SS;
if opt == 1
    df(1,2)=diff(ff(1),'omega')*omegabar;     df(2,2)=diff(ff(2),'omega')*omegabar;      df(3,2)=diff(ff(3),'omega')*omegabar;
elseif opt == 0
    df(1,2)=diff(ff(1),'Zp1')*Z_SS;     df(2,2)=diff(ff(2),'Zp1')*Z_SS;      df(3,2)=diff(ff(3),'Zp1')*Z_SS;
elseif opt == 2
    df(1,2)=diff(ff(1),'sp1')*sp1_SS;     df(2,2)=diff(ff(2),'sp1')*sp1_SS;  df(3,2)=diff(ff(3),'sp1')*sp1_SS;
end
ia=3;
if opt ~= 2
    ia=5;
    df(1,3)=diff(ff(1),'Rkp1')*Rk;      df(2,3)=diff(ff(2),'Rkp1')*Rk;       df(3,3)=diff(ff(3),'Rkp1')*Rk;
    df(1,4)=diff(ff(1),'Rep1')*Re;      df(2,4)=diff(ff(2),'Rep1')*Re;       df(3,4)=diff(ff(3),'Rep1')*Re;
end
df(1,ia)=diff(ff(1),'lev')*lev_SS;   df(2,ia)=diff(ff(2),'lev')*lev_SS;df(3,ia)=diff(ff(3),'lev')*lev_SS;
if opt ~= 2
    df(1,ia+1)=diff(ff(1),'Petil')*Pe_SS;     df(2,ia+1)=diff(ff(2),'Petil')*Pe_SS;      df(3,ia+1)=diff(ff(3),'Petil')*Pe_SS;
elseif opt == 2
    df(1,ia+1)=diff(ff(1),'ZminusR')*ZminusR_SS;     df(2,ia+1)=diff(ff(2),'ZminusR')*ZminusR_SS;    df(3,ia+1)=diff(ff(3),'ZminusR')*ZminusR_SS;
    df(1,ia+2)=diff(ff(1),'Petil')*Pe_SS;     df(2,ia+2)=diff(ff(2),'Petil')*Pe_SS;      df(3,ia+2)=diff(ff(3),'Petil')*Pe_SS;
end
edf=eval(df);

%order in note: ZminusR lev sig sp1 Ptilde
%order in computations above: sig sp1 lev ZminusR Ptilde

A=[edf([1:2],4) edf([1:2],3)];
B=[edf([1:2],1) edf([1:2],2)];
C=-A\B;

%[Pe-hat(t+1) lev(t)]'=C*[sig-hat(t) s-hat(t+1)]'
%Petilde-hat(t+1) = a(1)*sig-hat(t) + a(2)*sp1(t) + a(3)*lev(t) + a(4)*ZminusR(t)

a=-edf(3,:)/edf(3,end);
a1=[a(4) a(3)];
a2=[a(1) a(2)];
d=a1*C+a2;

%the following should be zero in the stochastic simulations:
%edf*[sig-hat Zp1-hat Rkp1-hat Rep1-hat (q-hat+kp1-hat - np1hat) Petil-hat]' 
